library(clv)
source("evaluation_methods.r")

# define data file
# to use assault data, change 'theft' in the filename to 'assault'
unit_crime_file<-"theft_block_hour.csv"
unit_context_file<-"usf_theft.csv"


d<-read.csv(unit_crime_file)

week.index<-floor(d[,1]/7)+1

train.weeks<-80
train.week<-(80-train.weeks+1):80
train.data<-d[week.index %in% train.week,]
train.table<-as.matrix(table(train.data$grid.index, train.data$time.index))
train.table<-ifelse(train.table>5, 5, train.table)

test.week<-81:100
test.data<-d[week.index %in% test.week,]
test.table<-as.matrix(table(test.data$grid.index, test.data$time.index))
test.table<-ifelse(test.table>5, 5, test.table)


n<-dim(train.table)[1]
m<-dim(train.table)[2]

uc<-read.csv(unit_context_file, header=F)
uc<-uc[,c(1,2,4:ncol(uc))]
block<-character(nrow(uc))
tow<-character(nrow(uc))
for(i in 1:nrow(uc)){
  id<-strsplit(as.character(uc[i,1]), "\\|")
  block[i]<-id[[1]][2]
  tow[i]<-id[[1]][1]
}
unique_block<-unique(block)
train_data<-{}
for(i in 1:length(unique_block)){
  line<-colMeans(uc[block==unique_block[i],2:ncol(uc)])
  train_data<-rbind(train_data, line)
}
model<-glm(V2 ~., data=data.frame(train_data))
pred<-predict(model, data.frame(train_data)[,2:ncol(train_data)])
socb_data<-data.frame(block=unique_block, pred=pred)
socb_block<-rep(0, n)
for(i in 1:n){
  block_name<-rownames(train.table)[i]
  socb_block[i]<-socb_data[socb_data[,1] == block_name, 2]
}

unique_tow<-unique(tow)
train_data<-{}
for(i in 1:length(unique_tow)){
  line<-colMeans(uc[tow==unique_tow[i],2:ncol(uc)])
  train_data<-rbind(train_data, line)
}
model<-glm(V2 ~., data=data.frame(train_data))
pred<-predict(model, data.frame(train_data)[,2:ncol(train_data)])
socb_data<-data.frame(tow=unique_tow, pred=pred)
socb_tow<-rep(0, m)
for(j in 1:m){
  tow_name<-colnames(train.table)[j]
  socb_tow[j]<-socb_data[socb_data[,1] == tow_name, 2]
}

samples<-{}
for(i in 1:n){
  for(j in 1:m){
    if(train.table[i, j] !=0)
      samples<-rbind(samples, c(i, j, train.table[i,j]))
  }
}

K<-8
alpha<-0.01
beta_u<-0.01
beta_i<-2.5
n_iter<-50

pred_one<-function(i, j){
  p<-b_u[i]+b_i[j]+P[i, ] %*% Q[j,]
  return(p[1])
}
pred_one_b<-function(i, j){
  p<-socb_block[i]+socb_tow[j]+b_u[i]+b_i[j]+P[i, ] %*% Q[j,]
  return(p[1])
}

set.seed(123)
P<-matrix(rnorm(n*K), nrow=n, ncol=K)
Q<-matrix(rnorm(m*K), nrow=m, ncol=K)
b_u<-rep(0, n)
b_i<-rep(0, m)
for(iter in 1:n_iter){
  sum_e<-0
  for(l in 1:nrow(samples)){
    s<-samples[sample(nrow(samples), 1),]
    i<-s[1]
    j<-s[2]
    r<-s[3]
    p<-pred_one_b(i, j)
    e<-(r - p)
    b_u[i]<-b_u[i]<-alpha*(e - beta_u*b_u[i])
    b_i[j]<-b_i[j]<-alpha*(e - beta_i*b_i[j])
    P[i,]<-P[i,]+alpha*(e*Q[j,] - beta_u*P[i,])
    Q[j,]<-Q[j,]+alpha*(e*P[i,] - beta_i*Q[j,])
    sum_e<-sum_e+e*e
  }
  print(c("iter", iter))
  print(c("RMSE", sqrt(sum_e/nrow(samples))))
}

pred.table<-matrix(0, nrow=n, ncol=m)
for(i in 1:n){
  for(j in 1:m){
    pred.table[i,j]<-pred_one_b(i, j)
  }
}

colnames(pred.table)<-colnames(train.table)
rownames(pred.table)<-rownames(train.table)

eval(pred.table, train.table, test.table)
